# taken from bootsect.S
SETUPSEG = 0x9020

.code16
.text

.global _start
_start:
	.word   0x00eb, 0x00eb		# short jump delay
	movw    $0x9000, %ax
	movw    %ax, %ds			# %ds = 0x9000

# Ok, it's time to move in protected mode.
	cli							# no interrupts
	movb    $0x80, %al			# disable NMI bootup
	outb    %al, $0x70			# out[0x80] = 0x70

# Move the system from 0x1000:0 to 0x0100:0. In protected mode the memory is
# organized in pages of 4Kb, so we transfer the kernel from 0x1000:0 to 
# 0x0100:0 a page at a time.

	movw    $0x100, %ax
	movw    $0x1000, %bx
	cld							# move forward

do_move:
	movw    %ax, %es			# %es = destination address (first loop is 
								# 0x100)
    addw    $0x100, %ax			# %ax = %ax + 4Kb
    cmpw    $0x9000, %ax		# if (%ax == 0x9000) jump to end_move
    jz  end_move
    movw    %bx, %ds			# %ds = source address (first loop is 0x1000)
    addw    $0x100, %bx			# %bx = %bx + 4Kb
    subw    %di, %di			# %di = 0
    subw    %si, %si			# %si = 0
    movw    $0x800, %cx			# %cx = 0x800
    rep							#
    movsw						# copy 2048 words = 4096 bytes = 4Kb from
								# %ds:%si to %es:%di
    jmp do_move					# jump to do_move


end_move:
    movw    $SETUPSEG, %ax
    movw    %ax, %ds			# %ds = $SETUPSEG


# This is the memory situation after the copy.
#
#    ---------------
#   |               | 0x0000:0
#   |     ....      |
#   |_______________|
#   |     COPY      | 0x0100:0
#   |     IMAGE     |
#   |     KERNEL    | (size 0x7F00)
#   |    (main.c)   |
#   |               |
#   |_______________|
#   |               |
#   |     ....      |
#   |_______________|
#   |   BOOTSECTOR  | 0x07C0:0
#   |               |
#   |go: ....       |
#   |               |
#   |_______________|
#   |               |
#   |   .........   |
#   |_______________|
#   |               | 0x1000:0
#   |     IMAGE     |
#   |     KERNEL    | (size 0x7F00)
#   |    (main.c)   |
#   |_______________|
#   |   .........   |
#   |_______________|
#   |     COPY      |
#   |   BOOTSECTOR  | 0x9000:0
#   |               |
#   |go: ....       |
#   |               |
#   |_______________|
#   |    (head.S)   | 0x9000:0x200 = 0x90200
#   |               |
#   |     SETUP     | 4 sectors = 2048 bytes
#   |               |
#   |_______________|
#   |               |
#   |               |
#   |     STACK     |
#   |               |
#   |_______________| <- 0x9000:0x4000-12
#   |   DISK PARMS  |
#   |   (12 bytes)  |
#   |_______________| <- 0x9000:0x4000

# Print a message specifying that IDT and GTD are load.

	leaw    msg, %si		# now %ds:%si point to msg
	call    prtstr			# print the message

# Set the IDTR and GDTR registers
    lidt    idt_48          # base == 0, limit == 0
    lgdt    gdt_48          # limit == 2048 -> 256 entries

# Enable A20
    call    empty_8042
    movb    $0xd1, %al
    outb    %al, $0x64      # out[0xD1] = 0x64
    call    empty_8042
    movb    $0xdf, %al
    outb    %al, $0x60      # out[0xD1] = 0x60
    call    empty_8042

# reset coprocessor
    xorw    %ax, %ax
    outb    %al, $0xf0      # out[0] = 0xf0
    call    delay
    outb    %al, $0xf1      # out[0] = 0xf1
    call    delay

# Reprogram the Interrupts
    movb    $0x11, %al      # out[0x11] = 0x20
    outb    %al, $0x20
    call    delay

    outb    %al, $0xa0      # out[0x11] = 0xa0
    call    delay

    movb    $0x20, %al      # out[0x20] = 0x21 -> start of hardware ints
    outb    %al, $0x21
    call    delay

    movb    $0x28, %al      # out[0x28] = 0xa1 -> start of hardware ints2
    outb    %al, $0xa1
    call    delay

    movb    $0x04, %al      # out[0x04] = 0x21 -> 8259 is master
    outb    %al, $0x21
    call    delay


    movb    $0x02, %al      # out[0x02] = 0xa1 -> 8259 is slave
    outb    %al, $0xa1
    call    delay

    movb    $0x01, %al      # out[0x01] = 0x21 -> 8086 mode for both
    outb    %al, $0x21
    call    delay

    outb    %al, $0xa1      # out[0x01] = 0xa1
    call    delay

    movb    $0xff, %al      # out[0xff] = 0xa1 -> mask off all interrupts
    outb    %al, $0xa1
    call    delay

    movb    $0xfb, %al      # out[0xfb] = 0x21
    outb    %al, $0x21

# Switch in Protected mode setting the bit 0 of the Machine Status Word (MSW)
    movw    $0x0001, %ax
    lmsw    %ax
    call    delay

	.byte   0x66, 0xea		# prefix + jmpi-opcode
code32: .long   0x0			# will be set to 0x100000
		.word	8

done:
	jmp done

# empty_8042
# This routine empty the keyboard command queue.

empty_8042:
    call    delay           # a small delay
    inb $0x64, %al          # %al = in[0x64]
    testb   $0x1, %al
    jz  no_output           # if (bit %al[0] == 0) jump no_output
    call    delay           # a small delay
    inb $0x60, %al          # %al = in[0x60]
    jmp empty_8042          # jump to empty_8042
no_output:
    testb   $2, %al         # if (bit %al[1] == 1) jump empty_8042
    jnz empty_8042
    ret

delay:
    .word   0x00eb      # make a short jump call jmp $+2
    ret

prtstr: lodsb				# load the character to print from %ds:%si to %ax
        andb    %al,%al
        jz      fin			# print the character until %al is 0x0
        call    prnt1		# print the character at %ds:%si
        jmp     prtstr
fin:    ret

prnt1:						# use the service 0x0E of interrupt 0x10 to print
							# the character in %al.
		pushw   %ax		
        pushw   %cx
        xorb    %bh,%bh
        movw    $0x01, %cx
        movb    $0x0e, %ah
        int     $0x10
        popw    %cx
        popw    %ax
        ret

gdt:
        .word   0, 0, 0, 0                      # dummy
        .word   0, 0, 0, 0                      # unused

        .word   0xFFFF                          # 4Gb - (0x100000*0x1000 = 4Gb)
        .word   0                               # base address = 0
        .word   0x9A00                          # code read/exec
        .word   0x00CF                          # granularity = 4096, 386
                                                #  (+5th nibble of limit)

        .word   0xFFFF                          # 4Gb - (0x100000*0x1000 = 4Gb)
        .word   0                               # base address = 0
        .word   0x9200                          # data read/write
        .word   0x00CF                          # granularity = 4096, 386
                                                #  (+5th nibble of limit)

idt_48:
        .word   0                               # idt limit = 0
        .word   0, 0                            # idt base = 0L
gdt_48:
        .word   0x8000							# gdt limit=2048,
                                                #  256 GDT entries

msg:	.byte 13, 10
		.ascii "Loading IDT and GTD"
		.byte 0x0
